﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2019 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using System.Collections.Generic;

namespace HeuristicLab.Algorithms.EvolvmentModelsOfModels {
  [Item("NetworkMap", "A map of models of models of models")]
  [StorableType("C200ECC2-6D33-4468-A538-580B07D75B3C")]
  public class EMMNetworkMap : EMMMapBase<ISymbolicExpressionTree> {
    public int NeghboorNumber { get; set; }
    #region conctructors
    [StorableConstructor]
    protected EMMNetworkMap(StorableConstructorFlag _) : base(_) { }
    public EMMNetworkMap() : this(1) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EMMNetworkMap(this, cloner);
    }
    public EMMNetworkMap(int neghboorNumber = 10) : base() {
      NeghboorNumber = neghboorNumber;
    }
    public EMMNetworkMap(EMMNetworkMap original, Cloner cloner) : base(original, cloner) { NeghboorNumber = original.NeghboorNumber; }
    //public EMMNetworkMap(IRandom random, IEnumerable<ISymbolicExpressionTree> trees, int k, int neghboorNumber = 10) : this(neghboorNumber) {
    //  // constructor that should be used in case of creation of a new map from the start point
    //  ModelSet = trees.ToList();
    //  CalculateDistances();
    //  NeghboorNumber = neghboorNumber; // this shoud be done before map creation, so it is impossible to create tree wisout it
    //  CreateMap(random, k);
    //}
    //public EMMNetworkMap(IRandom random, IEnumerable<ISymbolicExpressionTree> trees, string fileName = "Map.txt") : base(random, trees, fileName) {
    //  NeghboorNumber = Map[0].Count;
    //}
    #endregion
    #region MapTransformation
    override public void CreateMap(IRandom random, int k) {
      K = Map.Count;
      ApplyNetworkMapCreationAlgorithm(random, Distances, Map, K, NeghboorNumber);
      for (int i = 0; i < Map.Count; i++) {
        ClusterNumber.Add(i);
      }
    }
    public static void ApplyNetworkMapCreationAlgorithm(IRandom random, double[,] distances, List<List<int>> map, int k, int neghboorNumber = 10) {
      int mapSize = distances.GetLength(0);
      List<double> currentList = new List<double>();
      for (int i = 0; i < mapSize; i++) {
        map.Add(new List<int>());
        for (int j = 0; j < mapSize; j++) {
          currentList.Add(distances[i, j]);
        }
        map[i].Add(HelpFunctions.ChooseMinElementIndex(currentList));
        while (map[i].Count < neghboorNumber) {
          map[i].Add(HelpFunctions.ChooseMinElementIndex(currentList, i, map[i]));
        }
        currentList.Clear();
      }
    }
    #endregion
    #region Dialog with surroudings
    override public ISymbolicExpressionTree NewModelForInizializtionNotTree(IRandom random, out int cluster, out int treeNumber) {
      var newTree = NewModelForInizializtion(random, out cluster, out treeNumber);
      return newTree;
    }
    #endregion
  }
}
